
###############
## CONSTANTS ##
###############

WALL     = -1
MINOTAUR = 2
SPACE    = 0
WALKED_1 = 1
WALKED_2 = 2

START_X = 0
START_Y = 1
END_X   = -1
END_Y   = -2


###########################
## THE ACTUAL ASSIGNMENT ##
###########################

def flood_fill(maze, x, y, a, b):
  maze[y][x] = b
  for times in range(len(max(maze, maze[0]))):
    for i in range(1, len(maze)):
      for j in range(1, len(maze[0])):
        if maze[i-1][j] == a or maze[i][j-1] == a or \
           maze[i-1][j] == b or maze[i][j-1] == b:
          if maze[i][j] == a:
            maze[i][j] = b

def maze_solve(maze):
  flood_fill(maze, START_X, START_Y, 0, 1)
  if maze[END_Y][END_X] == WALKED_1:
    return "ALIVE"
  flood_fill(maze, START_X, START_Y, 1, 2)
  flood_fill(maze, START_X, START_Y, 0, 2)
  if maze[END_Y][END_X] == WALKED_2:
    return "DEAD"
  else:
    return "TRAPPED"


###############################################
## ADDITIONAL CODE TO GENERATE A RANDOM MAZE ##
###############################################

# Source:
# http://en.wikipedia.org/wiki/Maze_generation_algorithm#Python_code_example

def random_maze(width=81, height=51, complexity=.75, density=.75):
    from random import randint
    # Only odd shapes
    shape = ((height // 2) * 2 + 1, (width // 2) * 2 + 1)
    # Adjust complexity and density relative to maze size
    complexity = int(complexity * (5 * (shape[0] + shape[1])))
    density    = int(density * (shape[0] // 2 * shape[1] // 2))
    # Build actual maze
    Z = [ [0]*shape[1] for i in range(shape[0]) ]
    # Fill borders
    for i in range(shape[0]):
      Z[i][0] = Z[i][-1] = 1
    for j in range(shape[1]):
      Z[0][j] = Z[-1][j] = 1
    # Make isles
    for i in range(density):
        x, y = randint(0, shape[1] // 2) * 2, randint(0, shape[0] // 2) * 2
        Z[y][x] = 1
        for j in range(complexity):
            neighbours = []
            if x > 1:             neighbours.append((y, x - 2))
            if x < shape[1] - 2:  neighbours.append((y, x + 2))
            if y > 1:             neighbours.append((y - 2, x))
            if y < shape[0] - 2:  neighbours.append((y + 2, x))
            if len(neighbours):
                y_,x_ = neighbours[randint(0, len(neighbours) - 1)]
                if Z[y_][x_] == 0:
                    Z[y_][x_] = 1
                    Z[y_ + (y - y_) // 2][x_ + (x - x_) // 2] = 1
                    x, y = x_, y_
    # Entrance and exit
    Z[START_Y][START_X] = 0
    Z[END_Y][END_X] = 0
    return Z

def maze_to_string(Z):
  s = ""
  for line in Z:
    for c in line:
      if c:
        s += "#"
      else:
        s += " "
    s += "\n"
  return s

def random_maze_file(width, height):
  from StringIO import StringIO
  f = StringIO(maze_to_string(random_maze(width, height)))
  return f

def read_maze_from_file(file):
  maze = []
  lines = file.readlines()
  for line in lines:
    new_l = []
    for ch in line:
      if ch == "#":
        new_l.append(WALL)
      elif ch == "M":
        new_l.append(MINOTAUR)
      elif ch == " ":
        new_l.append(SPACE)
    maze.append(new_l)

  return maze


####################################
## ADDITIONAL CODE TO DRAW A MAZE ##
####################################

def draw_maze_pyplot(maze):
  import matplotlib.pyplot as pyplot
  pyplot.figure(figsize=(10, 5))
  pyplot.imshow(maze, cmap=pyplot.cm.binary, interpolation='nearest')
  pyplot.xticks([]), pyplot.yticks([])
  pyplot.show()

def square(side, filled=False):
  import turtle
  if filled:
    turtle.fill(True)
    for i in range(4):
      turtle.fd(side)
      turtle.rt(90)
    turtle.fill(False)

def draw_maze_turtle(maze):
  import turtle
  side = 10
  turtle.reset()
  turtle.speed("fastest")
  turtle.ht()
  for i in range(len(maze)):
    for j in range(len(maze[0])):
      square(side, maze[i][j] == WALL)
      turtle.pu()
      turtle.fd(side)
      turtle.pd()
    turtle.pu()
    turtle.rt(180)
    turtle.fd(len(maze[0])*side)
    turtle.lt(90)
    turtle.fd(side)
    turtle.lt(90)
    turtle.pd()
  turtle.done()
    
def draw_maze(maze):
  try:
    draw_maze_pyplot(maze)
  except:
    try:
      draw_maze_turtle(maze)
    except:
      return False
  return True
